<?php
namespace MaxAddonsPro;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Provider_Max_General extends \Bricks\Integrations\Dynamic_Data\Providers\Base {

	public function register_tags() {
		$tags = $this->get_tags_config();

		foreach ( $tags as $key => $tag ) {
			
			$this->tags[ $key ] = [
				'name'     => '{' . $key . '}',
				'label'    => $tag['label'],
				'group'    => $tag['group'],
				'provider' => $this->name
			];

			if ( ! empty( $tag['render'] ) ) {
				$this->tags[ $key ]['render'] = $tag['render'];
			}
		}
	}

	public function get_tags_config() {
		$mab_label  = ' [Max Addons]';

		$tags = [
			// Post terms
			'max_post_terms' => [
				'label' => esc_html__( 'Post terms', 'max-addons' ) . $mab_label,
				'group' => 'post'
			],

			// Post published time ago
			'max_post_published_ago' => [
				'label' => esc_html__( 'Post published time ago', 'max-addons' ) . $mab_label,
				'group' => 'post'
			],

			// Post modified time ago
			'max_post_modified_ago' => [
				'label' => esc_html__( 'Post modified time ago', 'max-addons' ) . $mab_label,
				'group' => 'post'
			],

			// Parent post title
			'max_parent_post_title' => [
				'label' => esc_html__( 'Parent post title', 'max-addons' ) . $mab_label,
				'group' => 'post'
			],

			// Parent post url
			'max_parent_post_url' => [
				'label' => esc_html__( 'Parent post URL', 'max-addons' ) . $mab_label,
				'group' => 'post'
			],

			// Author posts URL
			'max_author_posts_url' => [
				'label' => esc_html__( 'Author posts URL', 'max-addons' ) . $mab_label,
				'group' => 'author'
			],

			// Loop index
			'max_loop_index' => [
				'label' => esc_html__( 'Loop index', 'max-addons' ) . $mab_label,
				'group' => 'post'
			],

			// Menu item label
			'max_menu_item_label' => [
				'label' => esc_html__( 'Menu item label', 'max-addons' ) . $mab_label,
				'group' => esc_html__( 'WP menu', 'max-addons' ) . $mab_label
			],

			// Menu item title attribute
			'max_menu_item_title_attr' => [
				'label' => esc_html__( 'Menu item title attribute', 'max-addons' ) . $mab_label,
				'group' => esc_html__( 'WP menu', 'max-addons' ) . $mab_label
			],

			// Menu item description
			'max_menu_item_description' => [
				'label' => esc_html__( 'Menu item description', 'max-addons' ) . $mab_label,
				'group' => esc_html__( 'WP menu', 'max-addons' ) . $mab_label
			],

			// Menu item link
			'max_menu_item_url' => [
				'label' => esc_html__( 'Menu item link', 'max-addons' ) . $mab_label,
				'group' => esc_html__( 'WP menu', 'max-addons' ) . $mab_label
			],
		];

		return $tags;
	}

	/**
	 * Main function to render the tag value for WordPress provider
	 *
	 * @param [type] $tag
	 * @param [type] $post
	 * @param [type] $args
	 * @param [type] $context
	 * @return void
	 */
	public function get_tag_value( $tag, $post, $args, $context ) {
		
		$post_id = isset( $post->ID ) ? $post->ID : '';

		// STEP: Check for filter args
		$filters = $this->get_filters_from_args( $args );

		// STEP: Get the value
		$value = '';

		$render = isset( $this->tags[ $tag ]['render'] ) ? $this->tags[ $tag ]['render'] : $tag;

		switch ( $render ) {

			// Post terms
			case 'max_post_terms':
				$taxonomy = isset( $filters['meta_key'] ) ? $filters['meta_key'] : 'category';
				$value    = strip_tags( get_the_term_list( $post->ID, $taxonomy, '', ', ' ) );
				break;

			// Post published time ago
			case 'max_post_published_ago':
				$value = human_time_diff( get_post_time( 'U', false, $post, false ), current_time( 'timestamp' ) ) . ' ' . esc_html__( 'ago', 'max-addons' );
				break;

			// Post modified time ago
			case 'max_post_modified_ago':
				$value = human_time_diff( get_post_modified_time( 'U', false, $post, false ), current_time( 'timestamp' ) ) . ' ' . esc_html__( 'ago', 'max-addons' );
				break;

			// Parent post title
			case 'max_parent_post_title':
				if ( isset( $post->post_parent ) && $post->post_parent > 0 ) {
					$value = get_the_title( $post->post_parent );
				}
				break;

			// Parent post url
			case 'max_parent_post_url':
				if ( isset( $post->post_parent ) && $post->post_parent > 0 ) {
					$parent_post = get_post( $post->post_parent );
					$value       = get_permalink( $parent_post );
				}
				break;

			// Author posts URL
			case 'max_author_posts_url':
				$user  = isset( $post->post_author ) ? get_user_by( 'id', $post->post_author ) : false;
				$value = $user ? get_author_posts_url( $user->ID ) : '';
				break;

			// Loop index
			case 'max_loop_index':
				if ( method_exists( '\Bricks\Query', 'get_query_object' ) ) {
					$query_object = \Bricks\Query::get_query_object();

					if ( $query_object ) {
						$offset = isset( $filters['num_words'] ) ? $filters['num_words'] : 0;
						$value  = intval( $query_object::get_loop_index() ) + $offset;
					}
				}
				break;

			// Menu item label
			case 'max_menu_item_label':
				$loop_object = self::get_loop_object();

				if ( $loop_object ) {
					$value  = $loop_object->title;
				}
				break;

			// Menu item title attribute
			case 'max_menu_item_title_attr':
				$loop_object = self::get_loop_object();

				if ( $loop_object ) {
					$value  = $loop_object->attr_title;
				}
				break;

			// Menu item description
			case 'max_menu_item_description':
				$loop_object = self::get_loop_object();

				if ( $loop_object ) {
					$value  = $loop_object->description;
				}
				break;

			// Menu item link
			case 'max_menu_item_url':
				$loop_object = self::get_loop_object();

				if ( $loop_object ) {
					$value  = $loop_object->url;
				}
				break;
		}

		// STEP: Apply context (text, link, image, media)
		$value = $this->format_value_for_context( $value, $tag, $post_id, $filters, $context );

		return $value;
	}

	public static function get_loop_object() {
		if ( method_exists( '\Bricks\Query', 'get_loop_object' ) ) {
			$looping_query_id = \Bricks\Query::is_any_looping();

			if ( $looping_query_id ) {
				return \Bricks\Query::get_loop_object( $looping_query_id );
			}

			return false;
		}
	}
}
